@@ -406,8 +406,6 @@ jd11_index_reader(GPPort *port, CameraFilesystem *fs, GPContext *context) {
 	    return ret;
 	}
 	sprintf(fn,"image%02i.pgm",i);
-	gp_file_set_type (file, GP_FILE_TYPE_PREVIEW);
-	gp_file_set_name(file, fn);
 	gp_file_set_mime_type(file, GP_MIME_PGM);
 	gp_file_append(file, THUMBHEADER, strlen(THUMBHEADER));
 	src = indexbuf+(i*64*48);
@@ -427,15 +425,14 @@ jd11_index_reader(GPPort *port, CameraFilesystem *fs, GPContext *context) {
 		gp_file_free (file);
 		return ret;
 	}
-	ret = gp_filesystem_set_file_noop(fs, "/", file, context);
+	ret = gp_filesystem_set_file_noop(fs, "/", fn, GP_FILE_TYPE_PREVIEW, file, context);
 	if (ret != GP_OK) return ret;
 
 	/* we also get the fs info for free, so just set it */
-	info.file.fields = GP_FILE_INFO_TYPE | GP_FILE_INFO_NAME | 
+	info.file.fields = GP_FILE_INFO_TYPE |
 			GP_FILE_INFO_WIDTH | GP_FILE_INFO_HEIGHT | 
 			GP_FILE_INFO_SIZE;
 	strcpy(info.file.type,GP_MIME_PNM);
-	strcpy(info.file.name,fn);
 	info.file.width		= 640;
 	info.file.height	= 480;
 	info.file.size		= 640*480*3+strlen(IMGHEADER);
@@ -446,7 +443,7 @@ jd11_index_reader(GPPort *port, CameraFilesystem *fs, GPContext *context) {
 	info.preview.width	= 64;
 	info.preview.height	= 48;
 	info.preview.size	= 64*48+strlen(THUMBHEADER);
-	ret = gp_filesystem_set_info_noop(fs, "/", info, context);
+	ret = gp_filesystem_set_info_noop(fs, "/", fn, info, context);
     }
     free(indexbuf);
     return GP_OK;
