@@ -1,7 +1,7 @@
 /* usb.c
  *
  * Copyright (C) 2001-2004 Mariusz Woloszyn <emsi@ipartners.pl>
- * Copyright (C) 2003-2007 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright (C) 2003-2011 Marcus Meissner <marcus@jet.franken.de>
  * Copyright (C) 2006-2007 Linus Walleij <triad@df.lth.se>
  *
  * This library is free software; you can redistribute it and/or
@@ -61,7 +61,7 @@
 
 /* PTP2_FAST_TIMEOUT: how long (in milliseconds) we should wait for
  * an URB to come back on an interrupt endpoint */
-#define PTP2_FAST_TIMEOUT       50
+#define PTP2_FAST_TIMEOUT       100
 
 /* Pack / unpack functions */
 
@@ -104,7 +104,7 @@ uint16_t
 ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
 		  unsigned long size, PTPDataHandler *handler
 ) {
-	uint16_t ret;
+	uint16_t ret = PTP_RC_OK;
 	int res, wlen, datawlen;
 	PTPUSBBulkContainer usbdata;
 	unsigned long bytes_left_to_transfer, written;
@@ -273,7 +273,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 		if (dtoh16(usbdata.code)!=ptp->Code) {
 			/* A creative Zen device breaks down here, by leaving out
 			 * Code and Transaction ID */
-			if (MTP_ZEN_BROKEN_HEADER(camera->pl)) {
+			if (MTP_ZEN_BROKEN_HEADER(params)) {
 				gp_log (GP_LOG_DEBUG, "ptp2/ptp_usb_getdata", "Read broken PTP header (Code is %04x vs %04x), compensating.",
 					dtoh16(usbdata.code), ptp->Code
 				);
@@ -434,7 +434,6 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 {
 	uint16_t 		ret;
 	unsigned long		rlen;
-	Camera			*camera = ((PTPData *)params->data)->camera;
 	PTPUSBBulkContainer	usbresp;
 	/*GPContext		*context = ((PTPData *)params->data)->context;*/
 
@@ -453,7 +452,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 		ret = dtoh16(usbresp.code);
 	}
 	if (ret!=PTP_RC_OK) {
-		gp_log (GP_LOG_ERROR, "ptp2/usb_getresp","request code 0x%04x getting resp error 0x%04x", resp->Code, ret);
+		gp_log (GP_LOG_DEBUG, "ptp2/usb_getresp","request code 0x%04x getting resp error 0x%04x", resp->Code, ret);
 		return ret;
 	}
 	/* build an appropriate PTPContainer */
@@ -461,7 +460,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 	resp->SessionID=params->session_id;
 	resp->Transaction_ID=dtoh32(usbresp.trans_id);
 	if (resp->Transaction_ID != params->transaction_id - 1) {
-		if (MTP_ZEN_BROKEN_HEADER(camera->pl)) {
+		if (MTP_ZEN_BROKEN_HEADER(params)) {
 			gp_log (GP_LOG_DEBUG, "ptp2/ptp_usb_getresp", "Read broken PTP header (transid is %08x vs %08x), compensating.",
 				resp->Transaction_ID, params->transaction_id - 1
 			);
@@ -469,7 +468,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 		}
 		/* else will be handled by ptp.c as error. */
 	}
-
+	resp->Nparam=(rlen-12)/4;
 	resp->Param1=dtoh32(usbresp.payload.params.param1);
 	resp->Param2=dtoh32(usbresp.payload.params.param2);
 	resp->Param3=dtoh32(usbresp.payload.params.param3);
@@ -487,11 +486,16 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 static inline uint16_t
 ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 {
-	int			result, timeout;
+	int			result, timeout, fasttimeout;
 	unsigned long		rlen;
 	PTPUSBEventContainer	usbevent;
 	Camera			*camera = ((PTPData *)params->data)->camera;
 
+	if (params->deviceinfo.VendorExtensionID == PTP_VENDOR_CANON)
+		fasttimeout = PTP2_FAST_TIMEOUT*2;
+	else
+		fasttimeout = PTP2_FAST_TIMEOUT;
+
 	PTP_CNT_INIT(usbevent);
 
 	if (event==NULL)
@@ -504,8 +508,9 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 		break;
 	case PTP_EVENT_CHECK_FAST:
 		gp_port_get_timeout (camera->port, &timeout);
-		gp_port_set_timeout (camera->port, PTP2_FAST_TIMEOUT);
+		gp_port_set_timeout (camera->port, fasttimeout);
 		result = gp_port_check_int (camera->port, (char*)&usbevent, sizeof(usbevent));
+		if (result <= 0) result = gp_port_check_int (camera->port, (char*)&usbevent, sizeof(usbevent));
 		gp_port_set_timeout (camera->port, timeout);
 		break;
 	default:
@@ -517,6 +522,10 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 			return PTP_ERROR_TIMEOUT;
 		return PTP_ERROR_IO;
 	}
+	if (result == 0) {
+		gp_log (GP_LOG_DEBUG, "ptp2/usb_event", "reading event an 0 read occurred, assuming timeout.");
+		return PTP_ERROR_TIMEOUT;
+	}
 	rlen = result;
 	if (rlen < 8) {
 		gp_log (GP_LOG_ERROR, "ptp2/usb_event",
@@ -543,12 +552,13 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 	}
 	/* if we read anything over interrupt endpoint it must be an event */
 	/* build an appropriate PTPContainer */
-	event->Code=dtoh16(usbevent.code);
+	event->Nparam  = (rlen-12)/4;
+	event->Code   = dtoh16(usbevent.code);
 	event->SessionID=params->session_id;
 	event->Transaction_ID=dtoh32(usbevent.trans_id);
-	event->Param1=dtoh32(usbevent.param1);
-	event->Param2=dtoh32(usbevent.param2);
-	event->Param3=dtoh32(usbevent.param3);
+	event->Param1 = dtoh32(usbevent.param1);
+	event->Param2 = dtoh32(usbevent.param2);
+	event->Param3 = dtoh32(usbevent.param3);
 	return PTP_RC_OK;
 }
 
