@@ -25,9 +25,9 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <gphoto2-library.h>
-#include <gphoto2-result.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-result.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -54,7 +54,7 @@ static int
 g3_channel_read(GPPort *port, int *channel, char **buffer, int *len)
 {
 	unsigned char xbuf[0x800];
-	int ret;
+	int tocopy, ret, curlen;
 
 	ret = gp_port_read(port, xbuf, 0x800);
 	if (ret < GP_OK) { 
@@ -69,11 +69,26 @@ g3_channel_read(GPPort *port, int *channel, char **buffer, int *len)
 
 	*channel = xbuf[1];
 	*len = xbuf[4] + (xbuf[5]<<8) + (xbuf[6]<<16) + (xbuf[7]<<24);
+	/* Safety buffer of 0x800 ... we can only read in 0x800 chunks, 
+	 * otherwise the communication gets hickups. However *len might be
+	 * less.
+	 */
 	if (!*buffer)
-		*buffer = malloc(*len + 1);
+		*buffer = malloc(*len + 1 + 0x800);
 	else
-		*buffer = realloc(*buffer, *len + 1);
-	memcpy(*buffer, xbuf+8, *len);
+		*buffer = realloc(*buffer, *len + 1 + 0x800);
+	tocopy = *len;
+	if (tocopy > 0x800-8) tocopy = 0x800-8;
+	memcpy(*buffer, xbuf+8, tocopy);
+	curlen = tocopy;
+	while (curlen < *len) {
+		ret = gp_port_read(port, *buffer + curlen, 0x800);
+		if (ret < GP_OK) {
+			gp_log(GP_LOG_ERROR, "g3", "read error in g3_channel_read\n");
+			return ret;
+		}
+		curlen += ret;
+	}
 	(*buffer)[*len] = 0x00;
 	return GP_OK;
 }
@@ -92,18 +107,25 @@ g3_channel_read_bytes(
 		*buffer = realloc (*buffer, expected);
 	xbuf = malloc(65536 + 12);
 
-	id = gp_context_progress_start (context, expected, msg);
+	id = gp_context_progress_start (context, expected, "%s", msg);
 
 	/* The camera lets us read in packets of at least max 64kb size. */
 	while (expected > 0) {
 		int rest = expected;
 		if (rest > 65536) rest = 65536;
 
-		ret = gp_port_read(port, xbuf, (rest + 9 + 3) & ~3);
+		rest = (rest + 9 + 3) & ~3;
+		if (rest < 0x800) rest = 0x800;
+
+		ret = gp_port_read(port, xbuf, rest);
 		if (ret < GP_OK) {
 			gp_log(GP_LOG_ERROR, "g3", "read error in g3_channel_read\n");
 			return ret;
 		}
+		if (ret != rest) {
+			gp_log(GP_LOG_ERROR, "g3", "read error in g3_channel_read\n");
+			return ret;
+		}
 
 		if ((xbuf[2] != 0xff) || (xbuf[3] != 0xff)) {
 			gp_log(GP_LOG_ERROR, "g3", "first bytes do not match.\n");
@@ -148,7 +170,7 @@ g3_channel_write(GPPort *port, int channel, char *buffer, int len)
 		memcpy(xbuf+8, buffer + curlen, sendlen);
 		curlen += sendlen;
 		xbuf[8+sendlen] = 0x03;
-		ret = gp_port_write( port, xbuf, nlen);
+		ret = gp_port_write( port, (char*)xbuf, nlen);
 		free(xbuf);
 		if (ret < GP_OK) break;
 		len -= sendlen;
@@ -196,7 +218,7 @@ camera_abilities (CameraAbilitiesList *list)
 
 	memset(&a, 0, sizeof(a));
 	strcpy(a.model, "Ricoh:Caplio G3");
-	a.status	= GP_DRIVER_STATUS_TESTING;
+	a.status	= GP_DRIVER_STATUS_PRODUCTION;
 	a.port		= GP_PORT_USB;
 	a.usb_vendor	= 0x5ca;
 	a.usb_product	= 0x2204;
@@ -213,6 +235,82 @@ camera_abilities (CameraAbilitiesList *list)
 	a.usb_product	= 0x2202;
 	gp_abilities_list_append(list, a);
 
+	strcpy(a.model, "Ricoh:Caplio 300G");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2203;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Medion:MD 6126");
+	a.usb_vendor    = 0x5ca;
+	a.usb_product   = 0x2205;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio G4");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2208;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Capilo RX");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x220b;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio GX");
+	a.usb_vendor    = 0x5ca;
+	a.usb_product   = 0x220c;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio R1");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x220d;
+	gp_abilities_list_append(list, a);
+
+	/* same id as above */
+	strcpy(a.model, "Ricoh:Caplio RZ1");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x220d;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Sea & Sea:5000G");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x220e;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Rollei:dr5");
+	a.usb_vendor    = 0x5ca;
+	a.usb_product   = 0x220f;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio R1v");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2212;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio R2");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2213;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio GX 8");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2214;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio R3");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2216;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio R4");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2217;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio R5");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x221a;
+	gp_abilities_list_append(list, a);
+
 	return (GP_OK);
 }
 
@@ -309,6 +407,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	if (ret < GP_OK) goto out;
 	ret = g3_channel_read(camera->port, &channel, &reply, &len); /* reply */
 	if (ret < GP_OK) goto out;
+	gp_log(GP_LOG_DEBUG, "g3" , "reply %s", reply);
 	gp_file_set_data_and_size (file, buf, bytes);
 	buf = NULL; /* now owned by libgphoto2 filesystem */
 
@@ -323,20 +422,18 @@ out:
  * with it and sometimes refuses to send data the correct way
  */
 static int
-put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
+put_file_func (CameraFilesystem *fs, const char *folder, const char *fn, CameraFile *file,
 	       void *data, GPContext *context)
 {
 	Camera *camera = data;
 	char *buf = NULL, *reply = NULL, *cmd =NULL;
-	const char *fn = NULL, *imgdata = NULL;
+	const char *imgdata = NULL;
 	int ret, channel, len;
 	long size;
 
 	ret = g3_cwd_command (camera->port, folder);
 	if (ret < GP_OK) goto out;
 
-	ret = gp_file_get_name (file, &fn);
-	if (ret < GP_OK) goto out;
 	ret = gp_file_get_data_and_size (file, &imgdata, &size);
 	if (ret < GP_OK) goto out;
 
@@ -373,6 +470,7 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 		return ret;
 
 	cmd = malloc(strlen("DELE ")+strlen(filename)+1);
+	if (!cmd) return GP_ERROR_NO_MEMORY;
 	sprintf(cmd,"DELE %s",filename);
 	ret = g3_ftp_command_and_reply(camera->port, cmd, &reply);
 	if (ret < GP_OK)
@@ -400,6 +498,7 @@ rmdir_func (CameraFilesystem *fs, const char *folder,
 		return ret;
 
 	cmd = realloc(cmd,strlen("RMD ")+strlen(name)+1);
+	if (!cmd) return GP_ERROR_NO_MEMORY;
 	sprintf(cmd,"RMD %s",name);
 	ret = g3_ftp_command_and_reply(camera->port, cmd, &reply);
 	if (ret < GP_OK)
@@ -427,6 +526,7 @@ mkdir_func (CameraFilesystem *fs, const char *folder,
 		return ret;
 
 	cmd = realloc(cmd,strlen("MKD ")+strlen(name)+1);
+	if (!cmd) return GP_ERROR_NO_MEMORY;
 	sprintf(cmd,"MKD %s",name);
 	ret = g3_ftp_command_and_reply(camera->port, cmd, &reply);
 	if (ret < GP_OK)
@@ -526,10 +626,9 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	struct tm xtm;
 	char *cmd = NULL, *reply = NULL;
 
-	info->file.fields = GP_FILE_INFO_TYPE | GP_FILE_INFO_NAME |
+	info->file.fields = GP_FILE_INFO_TYPE |
 			GP_FILE_INFO_SIZE | GP_FILE_INFO_TYPE;
 	strcpy(info->file.type,GP_MIME_UNKNOWN);
-	strcpy(info->file.name, filename);
 
 	if (!strcmp(filename+9,"JPG") || !strcmp(filename+9,"jpg"))
 		strcpy(info->file.type,GP_MIME_JPEG);
@@ -561,21 +660,26 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		info->file.fields |= GP_FILE_INFO_MTIME;
 	}
 
-	sprintf(cmd, "-INFO %s/%s", folder,filename);
-	g3_ftp_command_and_reply(camera->port, cmd, &reply);
-	if (ret < GP_OK) goto out;
+	/* -INFO command only sometimes work on non jpeg/avi files */
+	if (	!strcmp(info->file.type,GP_MIME_JPEG) ||
+		!strcmp(info->file.type,GP_MIME_AVI)
+	) {
+		sprintf(cmd, "-INFO %s/%s", folder,filename);
+		ret = g3_ftp_command_and_reply(camera->port, cmd, &reply);
+		if (ret < GP_OK) goto out;
 
-	/* 200 1330313 byte W=2048 H=1536 K=0 for -INFO */
-	if (sscanf(reply, "200 %d byte W=%d H=%d K=%d for -INFO", &bytes, &width, &height , &k)) {
-		if (width != 0 && height != 0) {
-			info->file.fields |= GP_FILE_INFO_WIDTH | GP_FILE_INFO_HEIGHT;
-			info->file.height = height;
-			info->file.width = width;
+		/* 200 1330313 byte W=2048 H=1536 K=0 for -INFO */
+		if (sscanf(reply, "200 %d byte W=%d H=%d K=%d for -INFO", &bytes, &width, &height , &k)) {
+			if (width != 0 && height != 0) {
+				info->file.fields |= GP_FILE_INFO_WIDTH | GP_FILE_INFO_HEIGHT;
+				info->file.height = height;
+				info->file.width = width;
+			}
+			info->file.fields |= GP_FILE_INFO_SIZE;
+			info->file.size = bytes;
+			if (k != 0)
+				gp_log(GP_LOG_ERROR, "g3", "k is %d for %s/%s\n", k, folder,filename);
 		}
-		info->file.fields |= GP_FILE_INFO_SIZE;
-		info->file.size = bytes;
-		if (k != 0)
-			gp_log(GP_LOG_ERROR, "g3", "k is %d for %s/%s\n", k, folder,filename);
 	}
 out:
 	if (reply) free(reply);
@@ -584,16 +688,6 @@ out:
 	return (GP_OK);
 
 }
-static int
-set_info_func (CameraFilesystem *fs, const char *folder, const char *file,
-	       CameraFileInfo info, void *data, GPContext *context)
-{
-	/*Camera *camera = data;*/
-
-	/* Set the file info here from <info> */
-
-	return (GP_OK);
-}
 
 
 static int
@@ -602,15 +696,35 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 {
 	Camera *camera = data;
 	char *buf = NULL, *reply = NULL;
-	char *cmd;
-	int ret = GP_OK;
+	char *cmd = NULL;
+	int ret = GP_OK, channel, len, rlen;
 
 	if (!strcmp("/",folder)) {
-		/* Lets hope they dont invent more. */
-		/* NLST gives us the default folder back, 
-		 * we could check that too.
+		/* Lets hope they dont invent other names. */
+
+		/* We check the folder we get, since we should
+		 * not add EXT0 without SD card.
 		 */
-		gp_list_append (list, "EXT0", NULL);
+		ret = g3_ftp_command_and_reply(camera->port, "-NLST /", &buf);
+		if (ret < GP_OK) goto out;
+		if (buf[0] == '4') {
+			/* seen with R3: 450 Open Error... likely OK. */
+			goto out;
+		}
+		if (buf[0] != '1') {
+			ret = GP_ERROR_IO;
+			goto out;
+		}
+		ret = g3_channel_read(camera->port, &channel, &buf, &len); /* data. */
+		if (ret < GP_OK) goto out;
+		ret = g3_channel_read(camera->port, &channel, &reply, &rlen); /* next reply  */
+		if (ret < GP_OK) goto out;
+		gp_log(GP_LOG_DEBUG, "g3" , "reply %s", reply);
+
+		if (!strcmp("/EXT0",buf))
+			gp_list_append (list, "EXT0", NULL);
+		
+		/* always add internal memory */
 		gp_list_append (list, "IROM", NULL);
 		return GP_OK;
 	}
@@ -619,15 +733,17 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	strcpy(cmd, "-NLST ");
 	strcat(cmd, folder);
 
-	ret = g3_ftp_command_and_reply(camera->port, cmd, (char**)&buf);
+	ret = g3_ftp_command_and_reply(camera->port, cmd, &buf);
+	free(cmd);cmd = NULL;
 	if (ret < GP_OK) goto out;
 	if (buf[0] == '1') { /* 1xx means another reply follows */
-		int n = 0, channel, len, rlen;
+		int n = 0;
 
 		ret = g3_channel_read(camera->port, &channel, &buf, &len); /* data. */
 		if (ret < GP_OK) goto out;
 		g3_channel_read(camera->port, &channel, &reply, &rlen); /* next reply  */
 		if (ret < GP_OK) goto out;
+		gp_log(GP_LOG_DEBUG, "g3" , "reply %s", reply);
 
 		for (n=0;n<len/32;n++) {
 			if (buf[n*32+11] == 0x10) {
@@ -637,9 +753,11 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 				if (ret != GP_OK) goto out;
 			}
 		}
-		/* special case / folder, since we don't get the size data records */
 	} else {
-		ret = GP_ERROR_IO;
+		if (buf[0] == '4')
+			ret = GP_OK;
+		else	
+			ret = GP_ERROR_IO;
 	}
 out:
 	if (buf) free(buf);
@@ -647,6 +765,9 @@ out:
 	return ret;
 }
 
+/* for DOS FAT -> UNIX time conversion */
+static int day_n[] = { 0,31,59,90,120,151,181,212,243,273,304,334,0,0,0,0 };
+
 static int
 file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		void *data, GPContext *context)
@@ -654,6 +775,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	Camera *camera = data;
 	char *buf = NULL, *reply = NULL, *cmd;
 	int ret = GP_OK;
+	unsigned char *ubuf;
 
 	cmd = malloc(6 + strlen(folder) + 1);
 	strcpy(cmd, "-NLST ");
@@ -667,30 +789,90 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		if (ret < GP_OK) goto out;
 		g3_channel_read(camera->port, &channel, &reply, &rlen); /* next reply  */
 		if (ret < GP_OK) goto out;
+		gp_log(GP_LOG_DEBUG, "g3" , "reply %s", reply);
 
+		ubuf = (unsigned char*)buf;
 		for (n=0;n < len/32;n++) {
 			if (buf[n*32+11] == 0x20) {
-				char xfn[12];
+				CameraFileInfo  info;
+				char xfn[13];
+				int date, time, month, year;
 
-				strcpy(xfn, buf+n*32);
+				strncpy (xfn, buf+n*32, 8);
 				xfn[8] = '.';
-				strcpy(xfn+9, buf+n*32+8);
+				strncpy (xfn+9, buf+n*32+8, 3);
 				xfn[8+1+3] = '\0';
 
 				ret = gp_filesystem_append (fs, folder, xfn, context);
 				if (ret < GP_OK) goto out;
+
+				/* we also get parts of fs info for free, so just set it */
+				info.file.fields =
+						GP_FILE_INFO_SIZE |
+						GP_FILE_INFO_MTIME;
+				info.file.size =(ubuf[n*32+28]<<24)|
+						(ubuf[n*32+29]<<16)|
+						(ubuf[n*32+30]<< 8)|
+						(ubuf[n*32+31]    );
+				if (!strcmp(xfn+9,"JPG") || !strcmp(xfn+9,"jpg")) {
+					strcpy(info.file.type,GP_MIME_JPEG);
+					info.file.fields |= GP_FILE_INFO_TYPE;
+				}
+
+				if (!strcmp(xfn+9,"AVI") || !strcmp(xfn+9,"avi")) {
+					strcpy(info.file.type,GP_MIME_AVI);
+					info.file.fields |= GP_FILE_INFO_TYPE;
+				}
+
+				if (!strcmp(xfn+9,"WAV") || !strcmp(xfn+9,"wav")) {
+					strcpy(info.file.type,GP_MIME_WAV);
+					info.file.fields |= GP_FILE_INFO_TYPE;
+				}
+
+				if (!strcmp(xfn+9,"MTA") || !strcmp(xfn+9,"mta")) {
+					strcpy(info.file.type,"text/plain");
+					info.file.fields |= GP_FILE_INFO_TYPE;
+				}
+				info.preview.fields = 0;
+				date = (ubuf[n*32+16]) | (ubuf[n*32+17]<<8);
+				time = (ubuf[n*32+14]) | (ubuf[n*32+15]<<8);
+
+				/* from kernel fs/fat/, time_dos2unix. */
+        			month = ((date >> 5) - 1) & 15;
+				year = date >> 9;
+				info.file.mtime =
+					(time & 31)*2+60*((time >> 5) & 63)+
+					(time >> 11)*3600+86400*((date & 31)-1+
+					day_n[month]+(year/4)+year*365-
+					((year & 3) == 0 && month < 2 ? 1 : 0)+
+					3653);
+
+				ret = gp_filesystem_set_info_noop(fs, folder, xfn, info, context);
+ 
 			}
 		}
 	} else {
-		ret = GP_ERROR_IO;
+		if (buf[0] == '4') /* 450 Open Error ... like dir not there */
+			ret = GP_OK;
+		else		
+			ret = GP_ERROR_IO;
 	}
 out:
 	if (buf) free(buf);
 	if (reply) free(reply);
-	if (cmd) free(cmd);
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_file_func = get_file_func,
+	.del_file_func = delete_file_func,
+	.get_info_func = get_info_func,
+	.folder_list_func = folder_list_func,
+	.make_dir_func = mkdir_func,
+	.remove_dir_func = rmdir_func,
+};
+
 int
 camera_init (Camera *camera, GPContext *context) 
 {
@@ -702,15 +884,7 @@ camera_init (Camera *camera, GPContext *context)
         camera->functions->about                = camera_about;
 
 	/* Now, tell the filesystem where to get lists, files and info */
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func,
-				      folder_list_func, camera);
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func, set_info_func,
-				      camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-				      delete_file_func, camera);
-	gp_filesystem_set_folder_funcs (camera->fs, NULL, NULL,
-					mkdir_func, rmdir_func, camera);
-
+	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 
         gp_port_get_settings(camera->port, &settings);
 	settings.usb.inep = 0x81;
