@@ -7,10 +7,10 @@
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
@@ -28,7 +28,7 @@
 #include <gamma.h>
 
 
-#include <gphoto2.h>
+#include <gphoto2/gphoto2.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -46,15 +46,15 @@
 
 #include "sq905.h"
 
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2-port.h>
 
 #define GP_MODULE "sq905"
 
-struct {
-   	char *name;
+static const struct {
+	char *name;
 	CameraDriverStatus status;
-   	unsigned short idVendor;
-   	unsigned short idProduct;
+	unsigned short idVendor;
+	unsigned short idProduct;
 } models[] = {
 	{"SQ chip camera",    GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"Argus DC-1510",     GP_DRIVER_STATUS_PRODUCTION,   0x2770, 0x9120},
@@ -63,7 +63,7 @@ struct {
 	{"Mitek CD30P",       GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"GTW Electronics",   GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"Concord Eye-Q Easy",GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
-	{"Concord Eye-Q Duo", GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},	
+	{"Concord Eye-Q Duo", GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"Che-ez Snap",       GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"PockCam",           GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"Magpix B350",       GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
@@ -72,7 +72,8 @@ struct {
 	                      GP_DRIVER_STATUS_PRODUCTION , 0x2770 , 0x9120},
 	{"iConcepts digital camera" ,
                           GP_DRIVER_STATUS_PRODUCTION , 0x2770 , 0x9120},
-	{"Sakar Kidz Cam",    GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
+	{"Sakar Kidz Cam 86379",    GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770,
+								    0x9120},
 	{"ViviCam3350",       GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"ViviCam5B",         GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"DC-N130t",          GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
@@ -81,7 +82,9 @@ struct {
 	{"Shark SDC-519",     GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"Shark 2-in-1 Mini", GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{"Argus DC-1730",     GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x913c},
-	{"Request Ultra Slim", 
+	{"Request Ultra Slim",
+	                      GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
+	{"Global Point Splash Mini (underwater camera)",
 	                      GP_DRIVER_STATUS_EXPERIMENTAL, 0x2770, 0x9120},
 	{NULL,0,0}
 };
@@ -89,64 +92,66 @@ struct {
 int
 camera_id (CameraText *id)
 {
-    	strcpy (id->text, "SQ chipset camera");
-        
-    	return GP_OK;
+	strcpy (id->text, "SQ chipset camera");
+
+	return GP_OK;
 }
 
 
 int
 camera_abilities (CameraAbilitiesList *list)
 {
-    	int i;    
-    	CameraAbilities a;
+	int i;
+	CameraAbilities a;
 
-    	for (i = 0; models[i].name; i++) {
-        	memset (&a, 0, sizeof(a));
-       		strcpy (a.model, models[i].name);
-       		a.status = models[i].status;
-       		a.port   = GP_PORT_USB;
-       		a.speed[0] = 0;
-       		a.usb_vendor = models[i].idVendor;
-       		a.usb_product= models[i].idProduct;
-       		if (a.status == GP_DRIVER_STATUS_EXPERIMENTAL)
+	for (i = 0; models[i].name; i++) {
+		memset (&a, 0, sizeof(a));
+		strcpy (a.model, models[i].name);
+		a.status = models[i].status;
+		a.port   = GP_PORT_USB;
+		a.speed[0] = 0;
+		a.usb_vendor = models[i].idVendor;
+		a.usb_product= models[i].idProduct;
+		if (a.status == GP_DRIVER_STATUS_EXPERIMENTAL)
 			a.operations = GP_OPERATION_NONE;
 		else
 			a.operations = GP_OPERATION_CAPTURE_PREVIEW;
-       		a.folder_operations = GP_FOLDER_OPERATION_DELETE_ALL;
-		a.file_operations   = GP_FILE_OPERATION_PREVIEW + GP_FILE_OPERATION_RAW; 
-       		gp_abilities_list_append (list, a);
-    	}
+		a.folder_operations = GP_FOLDER_OPERATION_DELETE_ALL;
+		a.file_operations   = GP_FILE_OPERATION_PREVIEW
+					    + GP_FILE_OPERATION_RAW;
+		gp_abilities_list_append (list, a);
+	}
 
-    	return GP_OK;
+	return GP_OK;
 }
 
 static int
 camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 {
-    	sprintf (summary->text,_("Your USB camera has a S&Q chipset.\n" 
+	sprintf (summary->text,_("Your USB camera has a S&Q chipset.\n"
 				"The total number of pictures taken is %i\n"
 				"Some of these could be clips containing\n"
-				"several frames\n"), 
+				"several frames\n"),
 
-				camera->pl->nb_entries);  
+				camera->pl->nb_entries);
 
-    	return GP_OK;
+	return GP_OK;
 }
 
-static int camera_manual (Camera *camera, CameraText *manual, GPContext *context) 
+static int camera_manual (Camera *camera, CameraText *manual,
+						    GPContext *context)
 {
-	strcpy(manual->text, 
+	strcpy(manual->text,
 	_(
 	"For cameras with S&Q Technologies chip.\n"
 	"Should work with gtkam. Photos will be saved in PPM format.\n\n"
 
-	"All known S&Q cameras have two resolution settings. What\n" 
+	"All known S&Q cameras have two resolution settings. What\n"
 	"those are, will depend on your particular camera.\n"
 	"A few of these cameras allow deletion of all photos. Most do not.\n"
 	"Uploading of data to the camera is not supported.\n"
-	"The photo compression mode found on many of the S&Q\n" 
-	"cameras is supported, to some extent.\n" 
+	"The photo compression mode found on many of the S&Q\n"
+	"cameras is supported, to some extent.\n"
 	"If present on the camera, video clips are seen as subfolders.\n"
 	"Gtkam will download these separately. When clips are present\n"
 	"on the camera, there is a little triangle before the name of\n"
@@ -156,7 +161,7 @@ static int camera_manual (Camera *camera, CameraText *manual, GPContext *context
 	"be downloaded as separate photos, with special names which\n"
 	"specify from which clip they came. Thus, you may freely \n"
 	"choose to save clip frames in separate directories. or not.\n"
-	)); 
+	));
 
 	return (GP_OK);
 }
@@ -166,10 +171,10 @@ static int camera_manual (Camera *camera, CameraText *manual, GPContext *context
 static int
 camera_about (Camera *camera, CameraText *about, GPContext *context)
 {
-    	strcpy (about->text, _("sq905 generic driver\n"
+	strcpy (about->text, _("sq905 generic driver\n"
 			    "Theodore Kilgore <kilgota@auburn.edu>\n"));
 
-    	return GP_OK;
+	return GP_OK;
 }
 
 /*************** File and Downloading Functions *******************/
@@ -179,7 +184,7 @@ static int
 file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
                 void *data, GPContext *context)
 {
-        Camera *camera = data; 
+	Camera *camera = data; 
 	int i,n;
 	GP_DEBUG ("List files in %s\n", folder);
 	if (0==strcmp(folder, "/")) {
@@ -195,8 +200,10 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 			if (sq_is_clip(camera->pl, i)) n--;
 		}
 		i--;
-		if (!sq_is_clip(camera->pl, i)) return GP_ERROR_DIRECTORY_NOT_FOUND;
-		gp_list_populate(list, format, sq_get_num_frames(camera->pl, i));
+		if (!sq_is_clip(camera->pl, i))
+			return GP_ERROR_DIRECTORY_NOT_FOUND;
+		gp_list_populate(list, format,
+				    sq_get_num_frames(camera->pl, i));
 	}
 	return GP_OK;
 }
@@ -215,26 +222,16 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		gp_list_populate (list, "clip%03i", n);
 
 	}
-    	return GP_OK;
+	return GP_OK;
 }
 
 static int
 get_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 		      CameraFileInfo *info, void *data, GPContext *context)
 {
-	char path[1024];
-
-	if (strlen (folder) == 1)
-		snprintf (path, sizeof (path), "/%s", file);
-	else
-		snprintf (path, sizeof (path), "%s/%s", folder, file);
-
-        info->preview.fields = GP_FILE_INFO_NONE;
-        info->file.fields = GP_FILE_INFO_NAME; 
-
-        strcpy (info->file.name, file);
-
-        return (GP_OK);
+	memset (info, 0, sizeof(CameraFileInfo));
+	/* FIXME: fill in some stuff? */
+	return (GP_OK);
 }
 
 
@@ -243,15 +240,16 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       CameraFileType type, CameraFile *file, void *user_data,
 	       GPContext *context)
 {
-    	Camera *camera = user_data; 
+	Camera *camera = user_data;
 	int i, w, h, b, entry, frame, is_in_clip;
-	int nb_frames, to_fetch;  
+	int nb_frames, to_fetch;
 	int do_preprocess;
 	unsigned char comp_ratio;
-	unsigned char *frame_data, *rawdata; 
+	unsigned char *frame_data, *rawdata;
 	unsigned char *ppm, *ptr;
 	unsigned char gtable[256];
 	int size;
+	int this_cam_tile;
 
 	if (GP_FILE_TYPE_EXIF ==type) return GP_ERROR_FILE_EXISTS;
 
@@ -267,41 +265,48 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	if (0==strcmp(folder, "/")) {
 		i = atoi(filename+4);
 		do {
-			do entry++; 
-			while (sq_is_clip(camera->pl, entry) 
-					&& entry<camera->pl->nb_entries); 
+			do entry++;
+			while (sq_is_clip(camera->pl, entry)
+					&& entry<camera->pl->nb_entries);
 			i--;
 		} 
 		while (i>0);
-		if (entry == camera->pl->nb_entries) return GP_ERROR_FILE_NOT_FOUND;
+		if (entry == camera->pl->nb_entries)
+			return GP_ERROR_FILE_NOT_FOUND;
 		frame = 0;
 		is_in_clip = 0;
 	} else {
 		i = atoi(folder+1+4);
 		do {
-			do entry++; while (!sq_is_clip(camera->pl, entry) && entry<camera->pl->nb_entries);
+			do entry++; 
+			while (!sq_is_clip(camera->pl, entry) &&
+						entry<camera->pl->nb_entries);
 			i--;
 		} while (i>0);
-		if (entry == camera->pl->nb_entries) return GP_ERROR_DIRECTORY_NOT_FOUND;
+		if (entry == camera->pl->nb_entries)
+			return GP_ERROR_DIRECTORY_NOT_FOUND;
 		frame = atoi(filename+4)-1;
-		if (frame >= sq_get_num_frames(camera->pl, entry)) return GP_ERROR_FILE_NOT_FOUND;
+		if (frame >= sq_get_num_frames(camera->pl, entry))
+			return GP_ERROR_FILE_NOT_FOUND;
 		is_in_clip = 1;
 	}
-	
-	GP_DEBUG ("Download file %s from %s, entry = %d, frame = %d\n", 
+
+	GP_DEBUG ("Download file %s from %s, entry = %d, frame = %d\n",
 					    filename, folder, entry, frame);
 
-	/* Fetch entries until the one we need, and toss all but the one we need.
-	 * TODO: Either find out how to use the location info in the catalog to 
-	 * download just the entry needed, or show it is as impossible as it seems. 
+	/* Fetch entries until the one we need, and toss all before
+	 * the one we need.
+	 * TODO: Either find out how to use the location info in the catalog
+	 * to download just the entry needed, or show it is as impossible as
+	 * it seems to be.
 	 */
 
 	GP_DEBUG ("last entry was %d\n", camera->pl->last_fetched_entry);
 
 	/* Change register to DATA, but only if necessary */
 	if ((camera->pl->last_fetched_entry == -1) 
-	|| ((is_in_clip) && (frame == 0)) )	
-	
+	|| ((is_in_clip) && (frame == 0)) )
+
 	sq_access_reg(camera->port, DATA);
 
 	if (camera->pl->last_fetched_entry > entry) {
@@ -318,7 +323,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		nb_frames = sq_get_num_frames(camera->pl, to_fetch);
 		comp_ratio = sq_get_comp_ratio (camera->pl, to_fetch);
 		w = sq_get_picture_width (camera->pl, to_fetch);
-    		switch (w) {
+		switch (w) {
 		case 176: h = 144; break;
 		case 640: h = 480; break;
 		case 320: h = 240; break;
@@ -335,20 +340,20 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		}
 		GP_DEBUG("Fetch entry %i\n", to_fetch);
 		sq_read_picture_data 
-				(camera->port, camera->pl->last_fetched_data, b);
+			    (camera->port, camera->pl->last_fetched_data, b);
 		camera->pl->last_fetched_entry = to_fetch;
 	} while (camera->pl->last_fetched_entry<entry);
 
 	frame_data = camera->pl->last_fetched_data+(w*h)*frame/comp_ratio;
 	/* sq_preprocess ( ) turns the photo right-side-up and for some 
-	 * models must also de-mirror the photo 
+	 * models must also de-mirror the photo
 	 */
 
 	if (GP_FILE_TYPE_RAW!=type) {
 
 		if (do_preprocess) {
-			sq_preprocess(camera->pl->model, comp_ratio, is_in_clip, 
-				frame_data, w, h);
+			sq_preprocess(camera->pl->model, comp_ratio,
+					is_in_clip, frame_data, w, h);
 		}
 		
 		/*
@@ -356,54 +361,56 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		 */
 		ppm = malloc (w * h * 3 + 256); /* room for data + header */
 		if (!ppm) { return GP_ERROR_NO_MEMORY; }
-    		sprintf (ppm,
+		sprintf ((char *)ppm,
 			"P6\n"
 			"# CREATOR: gphoto2, SQ905 library\n"
 			"%d %d\n"
 			"255\n", w, h);
-		size = strlen (ppm);
+		size = strlen ((char *)ppm);
 		ptr = ppm + size;
-		if (comp_ratio>1) {
-			sq_decompress (camera->pl->model, ptr, frame_data, w, h, entry);			
-			sq_postprocess(camera->pl,w, h, ptr, entry);
-		}
-		size = size + (w * h * 3);
-		GP_DEBUG ("size = %i\n", size);
 
-		if ( comp_ratio == 1) {
 			switch (camera->pl->model) {
 			case SQ_MODEL_POCK_CAM:
-				gp_bayer_decode (frame_data, w , h , 
-						    ptr, BAYER_TILE_GBRG);
+			case SQ_MODEL_MAGPIX:
+				this_cam_tile = BAYER_TILE_GBRG;
 				break;
 			default:
-				gp_bayer_decode (frame_data, w , h , 
-						    ptr, BAYER_TILE_BGGR);
+				this_cam_tile = BAYER_TILE_BGGR;
 				break;
 			}
-			gp_gamma_fill_table (gtable, .5); 
-			gp_gamma_correct_single (gtable, ptr, w * h); 
-    		} 
+		size = size + (w * h * 3);
+		GP_DEBUG ("size = %i\n", size);
+		if (comp_ratio>1) {
+			rawdata = malloc (w*h);
+			if (!rawdata) return GP_ERROR_NO_MEMORY;
+			sq_decompress (camera->pl->model, rawdata,
+						frame_data, w, h);
+			gp_gamma_fill_table (gtable, .65); 
+		} else {
+			rawdata = frame_data;
+			gp_gamma_fill_table (gtable, .55);
+		}
+		gp_ahd_decode (rawdata, w , h , ptr, this_cam_tile);
+		gp_gamma_correct_single (gtable, ptr, w * h);
 
 		gp_file_set_mime_type (file, GP_MIME_PPM);
-    		gp_file_set_name (file, filename); 
-		gp_file_set_data_and_size (file, ppm, size);
+		gp_file_set_data_and_size (file, (char *)ppm, size);
 
 	} else {	/* type is GP_FILE_TYPE_RAW */
 		size = w*h/comp_ratio;
-		rawdata = malloc (size);
+		rawdata = malloc (size+16);
 		if (!rawdata) return GP_ERROR_NO_MEMORY;
 		memcpy (rawdata, frame_data, size);
+		memcpy (rawdata+size,camera->pl->catalog+16*entry,16);
 		gp_file_set_mime_type (file, GP_MIME_RAW);
-		gp_file_set_name (file, filename);
-	        gp_file_set_data_and_size (file, rawdata, size);  
+	        gp_file_set_data_and_size (file, (char *)rawdata, size+16);
 	}
 	/* Reset camera when done, for more graceful exit. */
-	if ((!(is_in_clip)&&(entry +1 == camera->pl->nb_entries)) 
+	if ((!(is_in_clip)&&(entry +1 == camera->pl->nb_entries))
 	|| ((is_in_clip)&& (frame +1 == nb_frames )))
 		sq_reset (camera->port);
 
-        return GP_OK;
+	return GP_OK;
 }
 
 static int
@@ -420,13 +427,10 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 
 static int
 camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
-
-
 {
-	unsigned char *frame_data; 
+	unsigned char *frame_data;
 	unsigned char *ppm, *ptr;
 	unsigned char gtable[256];
-	char filename[12] = "sq_cap.ppm";
 	int size;
 	int w = 320;
 	int h = 240;
@@ -444,15 +448,16 @@ camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 	sq_preprocess(camera->pl->model, 1, 0, frame_data, w, h);
 
 	/* Now put the data into a PPM image file. */
-	ppm = malloc (w * h * 3 + 256); 
-	if (!ppm) { return GP_ERROR_NO_MEMORY; }
-    	sprintf (ppm,
+	ppm = malloc (w * h * 3 + 256);
+	if (!ppm)
+		return GP_ERROR_NO_MEMORY;
+	sprintf ((char *)ppm,
 		"P6\n"
 		"# CREATOR: gphoto2, SQ905 library\n"
 		"%d %d\n"
 		"255\n", w, h);
-	ptr = ppm + strlen (ppm);	
-	size = strlen (ppm) + (w * h * 3);
+	ptr = ppm + strlen ((char*)ppm);
+	size = strlen ((char*)ppm) + (w * h * 3);
 	GP_DEBUG ("size = %i\n", size);
 	switch (camera->pl->model) {
 	case SQ_MODEL_POCK_CAM:
@@ -462,19 +467,17 @@ camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 		gp_bayer_decode (frame_data, w , h , ptr, BAYER_TILE_BGGR);
 		break;
 	}
-	
+
 	/* TO DO: 
-	 * Adapt postprocessing routine to work here, because results can   
-	 * vary greatly, depending both on lighting conditions and on 
+	 * Adapt some postprocessing routine to work here, because results
+	 * can vary greatly, depending both on lighting conditions and on
 	 * camera model.
 	 */
 
-	/* sq_postprocess(camera->pl,w, h, ptr, 1); */
-	gp_gamma_fill_table (gtable, .5); 
-	gp_gamma_correct_single (gtable, ptr, w * h); 
+	gp_gamma_fill_table (gtable, .5);
+	gp_gamma_correct_single (gtable, ptr, w * h);
 	gp_file_set_mime_type (file, GP_MIME_PPM);
-	gp_file_set_name (file, filename); 
-	gp_file_set_data_and_size (file, ppm, size);
+	gp_file_set_data_and_size (file, (char *)ppm, size);
 
 	sq_reset(camera->port);
         sq_access_reg(camera->port, CAPTURE);
@@ -515,22 +518,22 @@ camera_init(Camera *camera, GPContext *context)
 	int ret = 0;
 
 	/* First, set up all the function pointers */
-	camera->functions->summary      = camera_summary;
-        camera->functions->manual	= camera_manual;
-	camera->functions->about        = camera_about;
-	camera->functions->capture_preview	
+	camera->functions->summary	= camera_summary;
+	camera->functions->manual	= camera_manual;
+	camera->functions->about	= camera_about;
+	camera->functions->capture_preview
 					= camera_capture_preview;
-	camera->functions->exit	    	= camera_exit;
-   
+	camera->functions->exit		= camera_exit;
+
 	GP_DEBUG ("Initializing the camera\n");
 
 	ret = gp_port_get_settings(camera->port,&settings);
-	if (ret < 0) return ret; 
+	if (ret < 0) return ret;
  
 	ret = gp_port_set_settings(camera->port,settings);
-	if (ret < 0) return ret; 
+	if (ret < 0) return ret;
 
-        /* Tell the CameraFilesystem where to get lists from */
+	/* Tell the CameraFilesystem where to get lists from */
 	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 	camera->pl = malloc (sizeof (CameraPrivateLibrary));
 	if (!camera->pl) return GP_ERROR_NO_MEMORY;
@@ -547,6 +550,5 @@ camera_init(Camera *camera, GPContext *context)
 		return ret;
 	};
 
-
 	return GP_OK;
 }
