@@ -66,6 +66,8 @@ struct _GPPortPrivateLibrary {
 	int config;
 	int interface;
 	int altsetting;
+
+	int detached;
 };
 
 GPPortType
@@ -82,23 +84,17 @@ gp_port_library_list (GPPortInfoList *list)
 	struct usb_device *dev;
 	int nrofdevices = 0, i, i1, i2, unknownint;
 
-	/* default port first */
-	info.type = GP_PORT_USB;
-	strcpy (info.name, "Universal Serial Bus");
-	strcpy (info.path, "usb:");
-	CHECK (gp_port_info_list_append (list, info));
-
 	/* generic matcher. This will catch passed XXX,YYY entries for instance. */
-	memset (info.name, 0, sizeof(info.name));
-	strcpy (info.path, "^usb:");
+	gp_port_info_new (&info);
+	gp_port_info_set_type (info, GP_PORT_USB);
+	gp_port_info_set_name (info, "");
+	gp_port_info_set_path (info, "^usb:");
 	CHECK (gp_port_info_list_append (list, info));
 
 	usb_init ();
 	usb_find_busses ();
 	usb_find_devices ();
 
-	strcpy (info.name, "Universal Serial Bus");
-
 	bus = usb_get_busses();
 
 	/* Look and enumerate all USB ports. */
@@ -108,7 +104,8 @@ gp_port_library_list (GPPortInfoList *list)
 			if (	(dev->descriptor.bDeviceClass == USB_CLASS_HUB)		||
 				(dev->descriptor.bDeviceClass == USB_CLASS_HID)		||
 				(dev->descriptor.bDeviceClass == USB_CLASS_PRINTER)	||
-				(dev->descriptor.bDeviceClass == USB_CLASS_COMM)
+				(dev->descriptor.bDeviceClass == USB_CLASS_COMM)	||
+				(dev->descriptor.bDeviceClass == 0xe0)	/* wireless / bluetooth */
 			)
 				continue;
 			/* excepts HUBs, usually the interfaces have the classes, not
@@ -124,7 +121,9 @@ gp_port_library_list (GPPortInfoList *list)
 						struct usb_interface_descriptor *intf = &dev->config[i].interface[i1].altsetting[i2]; 
 						if (	(intf->bInterfaceClass == USB_CLASS_HID)	||
 							(intf->bInterfaceClass == USB_CLASS_PRINTER)	||
-							(intf->bInterfaceClass == USB_CLASS_COMM))
+							(intf->bInterfaceClass == USB_CLASS_COMM)	||
+							(intf->bInterfaceClass == 0xe0)	/* wireless/bluetooth*/
+						)
 							continue;
 						unknownint++;
 					}
@@ -140,11 +139,15 @@ gp_port_library_list (GPPortInfoList *list)
 		bus = bus->next;
 	}
 
+#if 0
 	/* If we already added usb:, and have 0 or 1 devices we have nothing to do.
 	 * This should be the standard use case.
 	 */
+	/* We never want to return just "usb:" ... also return "usb:XXX,YYY", and
+	 * let upper layers filter out the usb: */
 	if (nrofdevices <= 1) 
 		return (GP_OK);
+#endif
 
 	/* Redo the same bus/device walk, but now add the ports with usb:x,y notation,
 	 * so we can address all USB devices.
@@ -153,6 +156,7 @@ gp_port_library_list (GPPortInfoList *list)
 	while (bus) {
 		for (dev = bus->devices; dev; dev = dev->next) {
 			char *s;
+			char path[200];
 			/* Devices which are definitely not cameras. */
 			if (	(dev->descriptor.bDeviceClass == USB_CLASS_HUB)		||
 				(dev->descriptor.bDeviceClass == USB_CLASS_HID)		||
@@ -184,13 +188,26 @@ gp_port_library_list (GPPortInfoList *list)
 			/* Note: We do not skip USB storage. Some devices can support both,
 			 * and the Ricoh erronously reports it.
 			 */ 
-			sprintf (info.path, "usb:%s,%s", bus->dirname, dev->filename);
+			gp_port_info_new (&info);
+			gp_port_info_set_type (info, GP_PORT_USB);
+			gp_port_info_set_name (info, "Universal Serial Bus");
+			snprintf (path,sizeof(path), "usb:%s,%s", bus->dirname, dev->filename);
 			/* On MacOS X we might get usb:006,002-04a9-3139-00-00. */
-			s = strchr(info.path, '-');if (s) *s='\0';
+			s = strchr(path, '-');if (s) *s='\0';
+			gp_port_info_set_path (info, path);
 			CHECK (gp_port_info_list_append (list, info));
 		}
 		bus = bus->next;
 	}
+	/* This will only be added if no other device was ever added.
+	 * Users doing "usb:" usage will enter the regular expression matcher case. */
+	if (nrofdevices == 0) {
+		gp_port_info_new (&info);
+		gp_port_info_set_type (info, GP_PORT_USB);
+		gp_port_info_set_name (info, "Universal Serial Bus");
+		gp_port_info_set_path (info, "usb:");
+		CHECK (gp_port_info_list_append (list, info));
+	}
 	return (GP_OK);
 }
 
@@ -237,7 +254,9 @@ gp_port_usb_open (GPPort *port)
 	 */
 	port->pl->dh = usb_open (port->pl->d);
 	if (!port->pl->dh) {
-		gp_port_set_error (port, _("Could not open USB device (%m)."));
+		int saved_errno = errno;
+		gp_port_set_error (port, _("Could not open USB device (%s)."),
+				   strerror(saved_errno));
 		return GP_ERROR_IO;
 	}
 #if defined(LIBUSB_HAS_GET_DRIVER_NP) && defined(LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP)
@@ -245,7 +264,9 @@ gp_port_usb_open (GPPort *port)
 	ret = usb_get_driver_np (port->pl->dh, port->settings.usb.interface,
 		name, sizeof(name)
 	);
-	if (strstr(name,"usbfs")) {
+	if (name[0] != 0)
+		gp_log (GP_LOG_DEBUG,"libusb",_("Device has driver '%s' attached."), name);
+	if (strstr(name,"usbfs") || strstr(name,"storage")) {
 		/* other gphoto instance most likely */
 		gp_port_set_error (port, _("Camera is already in use."));
 		return GP_ERROR_IO_LOCK;
@@ -256,6 +277,8 @@ gp_port_usb_open (GPPort *port)
 		ret = usb_detach_kernel_driver_np (port->pl->dh, port->settings.usb.interface);
 		if (ret < 0)
 			gp_port_set_error (port, _("Could not detach kernel driver '%s' of camera device."),name);
+		else
+			port->pl->detached = 1;
 	} else {
 		if (errno != ENODATA) /* ENODATA - just no driver there */
 			gp_port_set_error (port, _("Could not query kernel driver of device."));
@@ -266,11 +289,15 @@ gp_port_usb_open (GPPort *port)
 	ret = usb_claim_interface (port->pl->dh,
 				   port->settings.usb.interface);
 	if (ret < 0) {
-		gp_port_set_error (port, _("Could not claim "
-			"interface %d (%m). Make sure no other program "
-			"or kernel module (such as %s) is using the device "
-			"and you have read/write access to the device."),
-			port->settings.usb.interface, "sdc2xx, stv680, spca50x");
+		int saved_errno = errno;
+		gp_port_set_error (port, _("Could not claim interface %d (%s). "
+					   "Make sure no other program "
+					   "or kernel module (such as %s) "
+					   "is using the device and you have "
+					   "read/write access to the device."),
+				   port->settings.usb.interface,
+				   strerror(saved_errno),
+				   "sdc2xx, stv680, spca50x");
 		return GP_ERROR_IO_USB_CLAIM;
 	}
 	return GP_OK;
@@ -284,26 +311,54 @@ gp_port_usb_close (GPPort *port)
 
 	if (usb_release_interface (port->pl->dh,
 				   port->settings.usb.interface) < 0) {
-		gp_port_set_error (port, _("Could not "
-			"release interface %d (%m)."),
-			port->settings.usb.interface);
+		int saved_errno = errno;
+		gp_port_set_error (port, _("Could not release "
+					   "interface %d (%s)."),
+				   port->settings.usb.interface,
+				   strerror(saved_errno));
 		return (GP_ERROR_IO);
 	}
 
 #if 0
-	/* This does break Canon EOS and some PowerShots ... -Marcus */
+	/* This confuses the EOS 5d camera and possible other EOSs. *sigh* */
 	/* This is only for our very special Canon cameras which need a good
 	 * whack after close, otherwise they get timeouts on reconnect.
 	 */
 	if (port->pl->d->descriptor.idVendor == 0x04a9) {
 		if (usb_reset (port->pl->dh) < 0) {
-			gp_port_set_error (port, _("Could not reset USB port (%m)."));
+			int saved_errno = errno;
+			gp_port_set_error (port, _("Could not reset "
+						   "USB port (%s)."),
+					   strerror(saved_errno));
 			return (GP_ERROR_IO);
 		}
 	}
 #endif
+#if defined(LIBUSB_HAS_GET_DRIVER_NP) && defined(LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP) && defined(USBDEVFS_CONNECT)
+	if (port->pl->detached) {
+		char filename[PATH_MAX + 1];
+		int fd;
+
+		/* FIXME shouldn't be a fixed path to usb root */
+		snprintf(filename, sizeof(filename) - 1, "%s/%s/%s", "/dev/bus/usb", port->pl->d->bus->dirname, port->pl->d->filename);
+		fd = open(filename, O_RDWR);
+
+		if (fd >= 0) {
+			struct usbdevfs_ioctl command;
+			command.ifno = 0;
+			command.ioctl_code = USBDEVFS_CONNECT;
+			command.data = NULL;
+			if (ioctl(fd, USBDEVFS_IOCTL, &command))
+				gp_log (GP_LOG_DEBUG,"libusb","reattach kernel driver failed");
+			close(fd);
+		}
+	}
+#endif
+
 	if (usb_close (port->pl->dh) < 0) {
-		gp_port_set_error (port, _("Could not close USB port (%m)."));
+		int saved_errno = errno;
+		gp_port_set_error (port, _("Could not close USB port (%s)."),
+				   strerror(saved_errno));
 		return (GP_ERROR_IO);
 	}
 
@@ -381,7 +436,7 @@ gp_port_usb_check_int (GPPort *port, char *bytes, int size, int timeout)
 	ret = usb_interrupt_read(port->pl->dh, port->settings.usb.intep,
 			     bytes, size, timeout);
         if (ret < 0) {
-		if (errno == EAGAIN)
+		if ((errno == EAGAIN) || (errno == ETIMEDOUT))
 			return GP_ERROR_TIMEOUT;
 		return GP_ERROR_IO_READ;
 	}
@@ -486,6 +541,9 @@ gp_port_usb_update (GPPort *port)
 {
 	int ret, ifacereleased = FALSE;
 
+	if (!port)
+		return GP_ERROR_BAD_PARAMETERS;
+
 	gp_log (GP_LOG_DEBUG, "libusb", "gp_port_usb_update(old int=%d, conf=%d, alt=%d), (new int=%d, conf=%d, alt=%d)",
 		port->settings.usb.interface,
 		port->settings.usb.config,
@@ -494,12 +552,12 @@ gp_port_usb_update (GPPort *port)
 		port->settings_pending.usb.config,
 		port->settings_pending.usb.altsetting
 	);
-	if (!port)
-		return GP_ERROR_BAD_PARAMETERS;
 
+#if 0
 	if (port->pl->interface == -1) port->pl->interface = port->settings.usb.interface;
 	if (port->pl->config == -1) port->pl->config = port->settings.usb.config;
 	if (port->pl->altsetting == -1) port->pl->altsetting = port->settings.usb.altsetting;
+#endif
 
 	/* The portname can also be changed with the device still fully closed. */
 	memcpy(&port->settings.usb.port, &port->settings_pending.usb.port,
@@ -519,14 +577,14 @@ gp_port_usb_update (GPPort *port)
 		);
 		if (usb_release_interface (port->pl->dh,
 					   port->pl->interface) < 0) {
-			gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","releasing the iface for config failed.");
+			gp_log (GP_LOG_DEBUG, "libusb","releasing the iface for config failed.");
 			/* Not a hard error for now. -Marcus */
 		} else {
 			gp_log (GP_LOG_DEBUG,"libusb","claiming interface %d", port->settings.usb.interface);
 			ret = usb_claim_interface (port->pl->dh,
 						   port->settings.usb.interface);
 			if (ret < 0) {
-				gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","reclaiming the iface for config failed.");
+				gp_log (GP_LOG_DEBUG, "libusb","reclaiming the iface for config failed.");
 				return GP_ERROR_IO_UPDATE;
 			}
 			port->pl->interface = port->settings.usb.interface;
@@ -542,7 +600,7 @@ gp_port_usb_update (GPPort *port)
 		 */
 		if (usb_release_interface (port->pl->dh,
 					   port->settings.usb.interface) < 0) {
-			gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","releasing the iface for config failed.");
+			gp_log (GP_LOG_DEBUG, "libusb","releasing the iface for config failed.");
 			ifacereleased = FALSE;
 		} else {
 			ifacereleased = TRUE;
@@ -551,16 +609,18 @@ gp_port_usb_update (GPPort *port)
 				     port->settings.usb.config);
 		if (ret < 0) {
 #if 0 /* setting the configuration failure is not fatal */
+			int saved_errno = errno;
 			gp_port_set_error (port,
-				_("Could not set config %d/%d (%m)"),
-				port->settings.usb.interface,
-				port->settings.usb.config);
+					   _("Could not set config %d/%d (%s)"),
+					   port->settings.usb.interface,
+					   port->settings.usb.config,
+					   strerror(saved_errno));
 			return GP_ERROR_IO_UPDATE;	
 #endif
-			gp_log (GP_LOG_ERROR, "gphoto2-port-usb","setting configuration from %d to %d failed with ret = %d, but continue...", port->pl->config, port->settings.usb.config, ret);
+			gp_log (GP_LOG_ERROR, "libusb","setting configuration from %d to %d failed with ret = %d, but continue...", port->pl->config, port->settings.usb.config, ret);
 		}
 
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-usb",
+		gp_log (GP_LOG_DEBUG, "libusb",
 			"Changed usb.config from %d to %d",
 			port->pl->config,
 			port->settings.usb.config);
@@ -570,7 +630,7 @@ gp_port_usb_update (GPPort *port)
 			ret = usb_claim_interface (port->pl->dh,
 						   port->settings.usb.interface);
 			if (ret < 0) {
-				gp_log (GP_LOG_DEBUG, "gphoto2-port-usb","reclaiming the iface for config failed.");
+				gp_log (GP_LOG_DEBUG, "libusb","reclaiming the iface for config failed.");
 			}
 		}
 		/*
@@ -584,15 +644,17 @@ gp_port_usb_update (GPPort *port)
 	if (port->settings.usb.altsetting != port->pl->altsetting) {
 		ret = usb_set_altinterface(port->pl->dh, port->settings.usb.altsetting);
 		if (ret < 0) {
-			gp_port_set_error (port, 
-				_("Could not set altsetting from %d "
-				"to %d (%m)"),
-				port->pl->altsetting,
-				port->settings.usb.altsetting);
+			int saved_errno = errno;
+			gp_port_set_error (port,
+					   _("Could not set altsetting from %d "
+					     "to %d (%s)"),
+					   port->pl->altsetting,
+					   port->settings.usb.altsetting,
+					   strerror(saved_errno));
 			return GP_ERROR_IO_UPDATE;
 		}
 
-		gp_log (GP_LOG_DEBUG, "gphoto2-port-usb",
+		gp_log (GP_LOG_DEBUG, "libusb",
 			"Changed usb.altsetting from %d to %d",
 			port->pl->altsetting,
 			port->settings.usb.altsetting);
@@ -698,7 +760,7 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 
 				port->pl->d = dev;
 
-				gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
+				gp_log (GP_LOG_VERBOSE, "libusb",
 					"Looking for USB device "
 					"(vendor 0x%x, product 0x%x)... found.", 
 					idvendor, idproduct);
@@ -712,7 +774,7 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 
 					if (dev->config[config].interface[interface].altsetting[altsetting].bInterfaceClass
 					    == USB_CLASS_MASS_STORAGE) {
-						gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
+						gp_log (GP_LOG_VERBOSE, "libusb",
 							_("USB device (vendor 0x%x, product 0x%x) is a mass"
 							  " storage device, and might not function with gphoto2."
 							  " Reference: %s"),
@@ -727,14 +789,14 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 					port->settings.usb.intep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_INTERRUPT);
 
 					port->settings.usb.maxpacketsize = 0;
-					gp_log (GP_LOG_DEBUG, "gphoto2-port-usb", "inep to look for is %02x", port->settings.usb.inep);
+					gp_log (GP_LOG_DEBUG, "libusb", "inep to look for is %02x", port->settings.usb.inep);
 					for (i=0;i<dev->config[config].interface[interface].altsetting[altsetting].bNumEndpoints;i++) {
 						if (port->settings.usb.inep == dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].bEndpointAddress) {
 							port->settings.usb.maxpacketsize = dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].wMaxPacketSize;
 							break;
 						}
 					}
-					gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
+					gp_log (GP_LOG_VERBOSE, "libusb",
 						"Detected defaults: config %d, "
 						"interface %d, altsetting %d, "
 						"inep %02x, outep %02x, intep %02x, "
@@ -755,9 +817,11 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 		}
 	}
 
+#if 0
 	gp_port_set_error (port, _("Could not find USB device "
 		"(vendor 0x%x, product 0x%x). Make sure this device "
 		"is connected to the computer."), idvendor, idproduct);
+#endif
 	return GP_ERROR_IO_USB_FIND;
 }
 
@@ -770,7 +834,7 @@ static int
 gp_port_usb_match_mtp_device(struct usb_device *dev,int *configno, int *interfaceno, int *altsettingno)
 {
 	char buf[1000], cmd;
-	int ret,i,i1,i2;
+	int ret,i,i1,i2, xifaces,xnocamifaces;
 	usb_dev_handle *devh;
 
 	/* All of them are "vendor specific" device class */
@@ -778,10 +842,70 @@ gp_port_usb_match_mtp_device(struct usb_device *dev,int *configno, int *interfac
 	if ((dev->descriptor.bDeviceClass!=0xff) && (dev->descriptor.bDeviceClass!=0))
 		return 0;
 #endif
+	if (dev->config) {
+		xifaces = xnocamifaces = 0;
+		for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
+			unsigned int j;
 
+			for (j = 0; j < dev->config[i].bNumInterfaces; j++) {
+				int k;
+				xifaces++;
+
+				for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
+					struct usb_interface_descriptor *intf = &dev->config[i].interface[j].altsetting[k]; 
+					if (	(intf->bInterfaceClass == USB_CLASS_HID)	||
+						(intf->bInterfaceClass == USB_CLASS_PRINTER)	||
+						(intf->bInterfaceClass == USB_CLASS_AUDIO)	||
+						(intf->bInterfaceClass == USB_CLASS_HUB)	||
+						(intf->bInterfaceClass == USB_CLASS_COMM)	||
+						(intf->bInterfaceClass == 0xe0)	/* wireless/bluetooth*/
+					)
+						xnocamifaces++;
+				}
+			}
+		}
+	}
+	if (xifaces == xnocamifaces) /* only non-camera ifaces */
+		return 0;
+
+	/* Marcus: Avoid this probing altogether, its too unstable on some devices */
+	return 0;
+
+#if 0
 	devh = usb_open (dev);
 	if (!devh)
 		return 0;
+
+	/*
+	 * Loop over the device configurations and interfaces. Nokia MTP-capable 
+	 * handsets (possibly others) typically have the string "MTP" in their 
+	 * MTP interface descriptions, that's how they can be detected, before
+	 * we try the more esoteric "OS descriptors" (below).
+	 */
+	if (dev->config) {
+		for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
+			unsigned int j;
+
+			for (j = 0; j < dev->config[i].bNumInterfaces; j++) {
+				int k;
+				for (k = 0; k < dev->config[i].interface[j].num_altsetting; k++) {
+					buf[0] = '\0';
+					ret = usb_get_string_simple(devh, 
+						dev->config[i].interface[j].altsetting[k].iInterface, 
+						(char *) buf, 
+						1024);
+					if (ret < 3)
+						continue;
+					if (strcmp((char *) buf, "MTP") == 0) {
+						gp_log (GP_LOG_DEBUG, "mtp matcher", "Configuration %d, interface %d, altsetting %d:\n", i, j, k);
+						gp_log (GP_LOG_DEBUG, "mtp matcher", "   Interface description contains the string \"MTP\"\n");
+						gp_log (GP_LOG_DEBUG, "mtp matcher", "   Device recognized as MTP, no further probing.\n");
+						goto found;
+					}
+				}
+			}
+		}
+	}
 	/* get string descriptor at 0xEE */
 	ret = usb_get_descriptor (devh, 0x03, 0xee, buf, sizeof(buf));
 	if (ret > 0) gp_log_data("get_MS_OSD",buf, ret);
@@ -814,6 +938,8 @@ gp_port_usb_match_mtp_device(struct usb_device *dev,int *configno, int *interfac
 		gp_log (GP_LOG_ERROR, "mtp matcher", "buf at 0x12 is %02x%02x%02x\n", buf[0x12], buf[0x13], buf[0x14]);
 		goto errout;
 	}
+
+found:
 	usb_close (devh);
 
 	/* Now chose a nice interface for us to use ... Just take the first. */
@@ -844,6 +970,7 @@ gp_port_usb_match_mtp_device(struct usb_device *dev,int *configno, int *interfac
 errout:
 	usb_close (devh);
 	return 0;
+#endif
 }
 
 static int
@@ -938,15 +1065,19 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 
 			if ((devname[0] != '\0') && strcmp(devname, dev->filename))
 				continue;
+			gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
+				"Looking for USB device "
+				"(class 0x%x, subclass, 0x%x, protocol 0x%x)...", 
+				class, subclass, protocol);
 
 			ret = gp_port_usb_match_device_by_class(dev, class, subclass, protocol, &config, &interface, &altsetting);
 			if (!ret)
 				continue;
 
 			port->pl->d = dev;
-			gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
-				"Looking for USB device "
-				"(class 0x%x, subclass, 0x%x, protocol 0x%x)... found.", 
+			gp_log (GP_LOG_VERBOSE, "libusb",
+				"Found USB class device "
+				"(class 0x%x, subclass, 0x%x, protocol 0x%x)", 
 				class, subclass, protocol);
 			/* Set the defaults */
 			if (dev->config) {
@@ -960,14 +1091,14 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 				port->settings.usb.outep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_OUT, USB_ENDPOINT_TYPE_BULK);
 				port->settings.usb.intep = gp_port_usb_find_ep(dev, config, interface, altsetting, USB_ENDPOINT_IN, USB_ENDPOINT_TYPE_INTERRUPT);
 				port->settings.usb.maxpacketsize = 0;
-				gp_log (GP_LOG_DEBUG, "gphoto2-port-usb", "inep to look for is %02x", port->settings.usb.inep);
+				gp_log (GP_LOG_DEBUG, "libusb", "inep to look for is %02x", port->settings.usb.inep);
 				for (i=0;i<dev->config[config].interface[interface].altsetting[altsetting].bNumEndpoints;i++) {
 					if (port->settings.usb.inep == dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].bEndpointAddress) {
 						port->settings.usb.maxpacketsize = dev->config[config].interface[interface].altsetting[altsetting].endpoint[i].wMaxPacketSize;
 						break;
 					}
 				}
-				gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
+				gp_log (GP_LOG_VERBOSE, "libusb",
 					"Detected defaults: config %d, "
 					"interface %d, altsetting %d, "
 					"idVendor ID %04x, idProduct %04x, "
@@ -986,10 +1117,11 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 			return GP_OK;
 		}
 	}
-
+#if 0
 	gp_port_set_error (port, _("Could not find USB device "
 		"(class 0x%x, subclass 0x%x, protocol 0x%x). Make sure this device "
 		"is connected to the computer."), class, subclass, protocol);
+#endif
 	return GP_ERROR_IO_USB_FIND;
 }
 
